(* ::Package:: *)

(************************************************************************)
(* This file was generated automatically by the Mathematica front end.  *)
(* It contains Initialization cells from a Notebook file, which         *)
(* typically will have the same name as this file except ending in      *)
(* ".nb" instead of ".m".                                               *)
(*                                                                      *)
(* This file is intended to be loaded into the Mathematica kernel using *)
(* the package loading commands Get or Needs.  Doing so is equivalent   *)
(* to using the Evaluate Initialization Cells menu command in the front *)
(* end.                                                                 *)
(*                                                                      *)
(* DO NOT EDIT THIS FILE.  This entire file is regenerated              *)
(* automatically each time the parent Notebook file is saved in the     *)
(* Mathematica front end.  Any changes you make to this file will be    *)
(* overwritten.                                                         *)
(************************************************************************)



(* ::Title:: *)
(*Pixel Arrays*)
(*Gray Scale Graphics and Data*)


(* ::Text:: *)
(*Sam Daniel*)
(*June 2006*)


(* ::Section:: *)
(*Graphics of Print or Record*)


(* ::Subsection:: *)
(*graphicsPrint*)


graphicsPrint[fingerID_]:=graphicsPrint[fingerID,imageFormat,400]
graphicsPrint[fingerID_,format_]:=graphicsPrint[fingerID,format,400]
graphicsPrint[fingerID_,format_,size_Integer]:=
Show[Import[fingerID<>"."<>format],PlotLabel->"\n"<>fingerID,ImageSize->size]

graphicsPrint[recordID_,fingerID_,format_]:=graphicsPrint[recordID,fingerID,format,400]
graphicsPrint[recordID_,fingerID_,format_,size_]:=
Module[{printFile=recordID<>"/"<>fingerID<>"."<>ImageFormat},
Show[Import[printFile],PlotLabel->"\n"<>fingerID,ImageSize->size]]


(* ::Subsection:: *)
(*graphicsPrintRecord*)


graphicsPrintRecord[printRecordID_,fingerIDs_,format_]:=
Module[{imageArray},
Print["Print Record : ",printRecordID<>"/",fingerIDs];
imageArray=Map[graphicsPrint[printRecordID,#,format,100]&,fingerIDs];
Grid[Partition[imageArray,5]]]


(* ::Section:: *)
(*Pixel Array Data, Processing, and Graphics of Print*)


(* ::Subsection:: *)
(*pixelArrayPrint*)


pixelArrayPrint[fingerID_,format_]:=graphicsPrint[fingerID,format][[1,1]]


pixelArrayPrint[recordID_,fingerID_,format_]:=graphicsPrint[recordID,fingerID,format][[1,1]]


(* ::Subsection:: *)
(*normalizePixelArray*)


normalizePixelArray=Compile[{{p,_Real,2},{n,_Integer}},
Module[{r,c,p0=Min[p],p1=Max[p],dp,dp0=0.05n},
dp=p1-p0;
If[dp<dp0,0p,Round[n (p-p0)/dp]]]];


(* ::Subsection:: *)
(*subPixelArray*)


subPixelArray[p_,n_,o_]:=
Module[{i0,j0,i1,j1},
{i0,j0}=o-Floor[n/2];
{i1,j1}={i0,j0}+n-1;
Take[p,{i0,i1},{j0,j1}]]


(* ::Subsection:: *)
(*padPixelArray*)


padPixelArray[p_,n_]:=padMatrix[p,n,0]
padPixelArray[p_,n_,z_]:=padMatrix[p,n,z]


padMatrix[m_,n_,p_]:=
Module[{d0,d1,d2},
d0=Dimensions[m];
d1=d0+n;
d2=d1+n;
PadRight[PadLeft[m,d1,p],d2,p]]


(* ::Subsection:: *)
(*trimPixelArray*)


trimPixelArray[p_,n_]:=trimMatrix[p,n]


trimMatrix[m_,n_]:=
Module[{t=(n+1){1,-1},d},
d=Dimensions[m];
If[Min[d-2n]<=0,
Print["Warning: Matrix trimming is excessive."],
Take[m,t,t]]]


(* ::Subsection:: *)
(*displayPixelArray*)


displayPixelArray[p_]:=Print[MatrixForm[Reverse[p]]]


(* ::Subsection:: *)
(*graphicsPixelArray*)


rasterPixelArray[p_]:=Raster[p/Max[p]]


graphicsPixelArray[p_]:=Graphics[Raster[p/Max[p]],AspectRatio->Automatic]


(* ::Section:: *)
(*Pixel Array Filtering*)


(* ::Subsection:: *)
(*Spatial Uniform LowPass, HighPass and BandPass Filtering*)


uniformBandPassPixelArray[n1_,0,par_]:=uniformLowPassPixelArray[n1,par]
uniformBandPassPixelArray[n1_,n2_,par_]:=
uniformLowPassPixelArray[n1,uniformHighPassPixelArray[n2,par]]
uniformHighPassPixelArray[n_,par_]:=par-uniformFilterPixelArray[n,par]
uniformLowPassPixelArray[n_,par_]:=uniformFilterPixelArray[n,par]


uniformFilterPixelArray[n_,par_]:=
Module[{m,u,p},
m=2n+1;
u=Table[1.0,{i,m},{j,m}]/m^2;
p=padPixelArray[par,n];
ListConvolve[u,p]]


(* ::Section:: *)
(*Pixel Array Cross Section Profile*)


ClearAll[crossSectionPrint]
crossSectionPrint[par_,{p1_,p2_}]:=crossSectionPrint[par,p1,p2]

crossSectionPrint[par_,p1_,p2_]:=
Module[{ns,pts,csp},
ns=Round[Norm[p2-p1]];
csp=If[ns<2,{},
pts=sampleSegmentPoints[p1,p2,ns];
Map[meanPixelValue[par,#]&,Map[Reverse@#&,pts]]];
csp-0Mean[csp]]

sampleSegmentPoints[p1_,p2_,ns_]:=
Module[{dp,ds},
dp=p2-p1;
ds=N[Norm[dp]];
dp=dp/ds;
Round[Map[dp #+p1&,Range[0,ds,ds/ns]]]]

meanPixelValue[par_,pt0_]:=
Module[{nbs,ij0,ijs,val,i,j},
nbs=pixelNeighbors;
ij0=Floor[pt0];
ijs=Map[ij0+#&,nbs];
val=Map[({i,j}=#;par[[i,j]])&,ijs];
Mean[val]]

pixelNeighbors={{0,0},{1,0},{0,1},{-1,0},{0,-1},{1,1},{-1,1},{-1,-1},{1,-1}};
pixelNeighbors={{0,0}};


(* ::Section:: *)
(*In Progress*)


(* ::Subsection:: *)
(*Pixel Array Contrast Enhancement*)


(* ::Text:: *)
(*The contrast of given r x c pixel array, par, may be enhanced by convolving a normalizing square window over it.  The typical size of the window should range from one to two ridge wavelengths.  Given that the window size is (2n + 1) x (2n + 1), the array will first be padded by n 0's all around.  Thus, contrast enahncement will yield a pixel array of the same size as the original.*)


(* ::Subsubsection:: *)
(*myBlockProcessing*)


myBlockProcessing[fun_,mat_,siz_,inc_]:=
Module[{par},
par=Partition[mat,siz,inc];
Map[fun,par,{2}]]


(* ::Subsubsection:: *)
(*contrastEnhancement*)


contrastEnhancement[par_,n_]:=
Module[{r,c,ppa,i},
{r,c}=Dimensions[par];
ppa=padPixelArray[par,n];
Map[(i=#;Map[normPixelArray[Take[ppa,{i,2n+i},{#,2n+#}]][[n+1,n+1]]&,Range[c]])&,Range[r]]]


contrastEnhancement[par_,n_]:=
Module[{ppa,blk},
ppa=padPixelArray[par,n];
myBlockProcessing[(normPixelArray[#][[n+1,n+1]]&),ppa,{2n+1,2n+1},{1,1}]]


(* ::Subsubsection::Closed:: *)
(*Example*)


(* ::Input:: *)
(*dataDir="/Users/sam/Desktop/SuperBackup/Biometrics/Fingerprints/Data/NIST27/Files/";*)
(*$Path=Union[Join[$Path,{dataDir}]];*)
(*fingerID="b101t9i_10";*)
(*ImageFormat="pgm";*)
(*par=pixelArrayPrint[fingerID,imageFormat]//N;*)
(*graphicsPixelArray[par]*)
(*n=8;*)
(*Timing[cpa=contrastEnhancement[par,n];]*)
(*graphicsPixelArray[cpa]*)


(* ::Input:: *)
(*Map[If[#<=160,0,255]&,cpa,{2}];*)
(*graphicsPixelArray[%]*)


(* ::Subsection::Closed:: *)
(*Spatial Percentile Filtering*)


(* ::Input:: *)
(*percentileFiltering[p_,n_,per_]:=*)
(*Module[{p0=n*per,p1=n (1-per)},*)
(*q=normalizePixelArray[p,n];*)
(*Map[If[#<p0,0,If[#>p1,n,#]]&,q,{2}]]*)


(* ::Subsection:: *)
(*Star Filtering ...*)


(* ::Subsubsection::Closed:: *)
(*Star Filter Bank*)


(* ::Input:: *)
(*starFilterBank[d_,n_]:=Map[buildStarFilter[#,d]&,starSpokes[d,n]]*)
(**)
(*starSpokes[d_,n_]:=*)
(*Module[{r=Floor[(d-1)/2],\[Theta]=Range[0,\[Pi]-\[Pi]/n,\[Pi]/n],pts},*)
(*pts=Round[Map[{Cos[#],Sin[#]}&,\[Theta]]r];*)
(*Map[lineSegment[{-#,#}+r+1]&,pts]]*)
(**)
(*starSpokes[d_,n_]:=*)
(*Module[{r=Floor[(d-1)/2],\[Theta]=Range[0,\[Pi]-\[Pi]/n,\[Pi]/n],pts},*)
(*pts=Round[Map[{Cos[#],Sin[#]}&,\[Theta]]r];*)
(*Map[lineSegment[{-#,#}]&,pts]]*)
(**)
(*buildStarFilter[ijs_,d_]:=(r=Floor[d/2];Table[If[MemberQ[ijs,{i,j}],1,0],{i,-r,r},{j,-r,r}])*)
(*starFilterBankGraphics[d_,n_]:=Map[plotGraphicsOne[#]&,starSpokes[d,n]]*)


(* ::Input:: *)
(*starSpokes[9,8]*)


(* ::Input:: *)
(*starFilterBank[d,n]*)


(* ::Input:: *)
(*d=9;n=8;*)
(*r=Floor[(d-1)/2]*)
(*\[Theta]=Range[0,\[Pi]-\[Pi]/n,\[Pi]/n]*)
(*pts=Round[Map[{Cos[#],Sin[#]}&,\[Theta]]r]*)
(*Map[lineSegment[{-#,#}]&,pts]*)


(* ::Input:: *)
(*Show[starFilterBankGraphics[9,8],PlotRange->{{-4,4},{-4,4}}]*)


(* ::Subsubsection::Closed:: *)
(*Standard Star Filter*)


(* ::Input:: *)
(*starFilter[p_,dia_,sfo_]:=*)
(*Module[{hs,ps,pps,meds,pds,best},*)
(*hs     =starFilterBank[dia,sfo];*)
(*ps     =Map[myListConvolve[#,p,255]&,hs];*)
(*pps   =Prepend[ps,trimArrayMargin[p,Floor[dia/2]]];*)
(*meds=Map[medianPixelArray[#]&,pps];*)
(*pds   =Map[pixelArrayCPD[#]&,pps];*)
(*best =CPDMatch[pds];*)
(*pps[[best]]]*)
(**)
(*CPDMatch[cpd_]:=Module[{cpd0=First[cpd],s0,ds,es},*)
(*s0=Apply[Plus,cpd0];*)
(*ds=Map[Abs[cpd0-#]&,Rest[cpd]];*)
(*es=Ceiling[Map[Apply[Plus,#]&,ds]1000/s0]/10.0;*)
(*(*Print["CPD % Error = ",Prepend[es,0]];*)*)
(*First[Flatten[Position[es,Min[es]]]]+1]*)


(* ::Input:: *)
(*dataDir="/Users/sam/Desktop/SuperBackup/Biometrics/Fingerprints/Data/NIST27/Files/";*)
(*$Path=Union[Join[$Path,{dataDir}]];*)
(*fingerID="b101t9i_10";*)
(*ImageFormat="pgm";*)
(*par=pixelArrayPrint[fingerID,imageFormat]//N;*)
(*p1=uniformBandPassPixelArray[2,8,par];*)
(*p1=Map[If[#<=4,0,255]&,p1,{2}];*)
(*graphicsPixelArray[p1]*)
(*p2=mvmSF[p1,9,8];*)
(*graphicsPixelArray[p2]*)
(*q2=Map[If[#<128,0,1]&,p1,{2}];*)
(*graphicsPixelArray[q2]*)



