(* Content-type: application/mathematica *)

(*** Wolfram Notebook File ***)
(* http://www.wolfram.com/nb *)

(* CreatedBy='Mathematica 6.0' *)

(*CacheID: 234*)
(* Internal cache information:
NotebookFileLineBreakTest
NotebookFileLineBreakTest
NotebookDataPosition[       145,          7]
NotebookDataLength[      6331,        204]
NotebookOptionsPosition[      4565,        136]
NotebookOutlinePosition[      6147,        196]
CellTagsIndexPosition[      6104,        193]
WindowFrame->Normal
ContainsDynamic->False*)

(* Beginning of Notebook Content *)
Notebook[{

Cell[CellGroupData[{
Cell[TextData[{
 StyleBox["Configuration",
  FontWeight->"Bold"],
 "\n",
 StyleBox["Configuration File",
  FontSize->16,
  FontWeight->"Bold"]
}], "Title",
 CellChangeTimes->{{3359497257.60821, 3359497361.33999}, 3359497449.95162, {
   3366509870.81328, 3366509873.16551}, {3366509917.62006, 
   3366509926.45961}, {3366561240.7763, 3366561289.60684}},
 TextAlignment->Center,
 TextJustification->0],

Cell["\<\
Sam Daniel
September 2006\
\>", "Text",
 CellChangeTimes->{{3359497205.94833`, 3359497210.41449`}, {3366509885.75482`,
   3366509890.72959`}},
 TextAlignment->Center,
 TextJustification->0],

Cell[CellGroupData[{

Cell["Preliminaries", "Section",
 CellChangeTimes->{{3359530989.26593, 3359531000.78687}, 3359578889.294, {
   3359594731.90523, 3359594745.64265}, {3365959051.93241, 3365959059.14291}}],

Cell[BoxData[{
 RowBox[{
  RowBox[{"mathDir", "=", 
   RowBox[{"sourceDir", "<>", "\"\</Math\>\""}]}], ";"}], "\n", 
 RowBox[{
  RowBox[{"dataDir", "=", 
   RowBox[{"sourceDir", "<>", "\"\</Data\>\""}]}], ";"}], "\n", 
 RowBox[{
  RowBox[{"filesDir", "=", 
   RowBox[{"dataDir", "<>", "\"\</NIST17/Files/\>\""}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"latentsDir", "=", 
   RowBox[{"dataDir", "<>", "\"\</NIST17/Latents/\>\""}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"$Path", "=", 
    RowBox[{"Union", "[", 
     RowBox[{"Join", "[", 
      RowBox[{"$Path", ",", 
       RowBox[{"{", 
        RowBox[{"mathDir", ",", "filesDir", ",", "latentsDir"}], "}"}]}], 
      "]"}], "]"}]}], ";"}], "\[IndentingNewLine]"}], "\[IndentingNewLine]", 
 RowBox[{"<<", "Minutiae.m"}], "\[IndentingNewLine]", 
 RowBox[{"<<", "PixelArrays.m"}], "\[IndentingNewLine]", 
 RowBox[{"<<", "RMM.m"}], "\[IndentingNewLine]", 
 RowBox[{"<<", "CrossSection.m"}], "\[IndentingNewLine]", 
 RowBox[{"<<", "Performance.m"}]}], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.36651003747756*^9, 3.36651004273935*^9}, 
   3.36651008783282*^9, {3.36651014347051*^9, 3.36651022490197*^9}, 
   3.36651057978118*^9, {3.36651067901981*^9, 3.36651082221126*^9}, {
   3.36651538691312*^9, 3.36651539261404*^9}, {3.3682909976572*^9, 
   3.36829102539095*^9}, {3.36829139374712*^9, 3.36829139614792*^9}, 
   3.36845641868118*^9, {3.36855203071014*^9, 3.36855204984713*^9}, {
   3.36855209601111*^9, 3.36855225902592*^9}, {3.3685528328604*^9, 
   3.36855283624038*^9}, {3.36855335381241*^9, 3.36855336454064*^9}, {
   3.36909575772105*^9, 3.36909576039812*^9}, 3.36917638818292*^9, {
   3.36933757756469*^9, 3.36933759697907*^9}, {3.36933782238166*^9, 
   3.36933782589911*^9}, {3.370739286268404*^9, 3.370739302616596*^9}, {
   3.3707395836321793`*^9, 3.370739634482642*^9}}],

Cell[BoxData[{
 RowBox[{
  RowBox[{"$RecursionLimit", "=", "10000"}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"$TextStyle", "=", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{"FontFamily", "\[Rule]", "\"\<Courier\>\""}], ",", 
      RowBox[{"FontSize", "\[Rule]", "8"}]}], "}"}]}], ";"}], 
  "\[IndentingNewLine]"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"imageFormat", "=", "\"\<pgm\>\""}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"minQuantum", "=", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{"16", ",", "16", ",", 
      RowBox[{"\[Pi]", "/", "6"}]}], "}"}], "//", "N"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"\[Theta]Scan", "=", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{"\[Theta]1", ",", "\[Theta]2", ",", "d\[Theta]"}], "}"}], "=", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{"-", "0.5"}], ",", "0.5", ",", "0.05"}], "}"}]}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"errorRadius", "=", "1.0"}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"processingStages", "=", 
   RowBox[{"{", 
    RowBox[{"\"\<  Registration\>\"", ",", "\"\< Matching\>\""}], "}"}]}], 
  ";"}]}], "Input",
 InitializationCell->True,
 CellChangeTimes->{3369095843.52701`}]
}, Open  ]]
}, Open  ]]
},
AutoGeneratedPackage->Automatic,
WindowToolbars->{"RulerBar", "EditBar"},
RulerUnits->"Inches",
WindowSize->{844, 639},
WindowMargins->{{Automatic, 58}, {Automatic, 2}},
PrintingCopies->1,
PrintingPageRange->{1, Automatic},
PageHeaders->{{
   Cell[
    TextData["Pixel Arrays \[LongDash] Gray Scale Graphics and Data"], 
    "Header"], None, 
   Cell[
    TextData["Daniel"], "Header"]}, {
   Cell[
    TextData["Daniel"], "Header"], None, 
   Cell[
    TextData["Pixel Arrays \[LongDash] Gray Scale Graphics and Data"], 
    "Header"]}},
PageFooters->{{
   Cell[
    TextData[{
      CounterBox["Page"]}], "PageNumber"], 
   Cell[
    TextData[""], "Footer"], 
   Cell[
    TextData[{
      ValueBox["FileName"]}], "Footer"]}, {
   Cell[
    TextData[{
      ValueBox["FileName"]}], "Footer"], 
   Cell[
    TextData[""], "Footer"], 
   Cell[
    TextData[{
      CounterBox["Page"]}], "PageNumber"]}},
PageHeaderLines->False,
PrintingOptions->{"FacingPages"->True,
"FirstPageHeader"->False,
"PaperOrientation"->"Portrait",
"PrintCellBrackets"->False,
"PrintMultipleHorizontalPages"->False,
"PrintRegistrationMarks"->True,
"PrintingMargins"->54},
ShowCellLabel->False,
Magnification->1.25,
GraphicsBoxOptions->{AlignmentPoint->ImageScaled[{0.5, 0.5}]},
GraphicsGridBoxOptions->{GraphicsSpacing->{0.1, 0.1}},
FrontEndVersion->"6.0 for Mac OS X PowerPC (32-bit) (October 11, 2006)",
StyleDefinitions->"MyNewDefault.nb"
]
(* End of Notebook Content *)

(* Internal cache information *)
(*CellTagsOutline
CellTagsIndex->{}
*)
(*CellTagsIndex
CellTagsIndex->{}
*)
(*NotebookFileOutline
Notebook[{
Cell[CellGroupData[{
Cell[590, 23, 399, 12, 144, "Title"],
Cell[992, 37, 199, 7, 63, "Text"],
Cell[CellGroupData[{
Cell[1216, 48, 186, 2, 87, "Section"],
Cell[1405, 52, 1892, 41, 220, "Input",
 InitializationCell->True],
Cell[3300, 95, 1237, 37, 163, "Input",
 InitializationCell->True]
}, Open  ]]
}, Open  ]]
}
]
*)

(* End of internal cache information *)
